import { parse, visit, } from 'graphql';
import picomatch from 'picomatch-browser';
import { getAutocompleteSuggestions, getDiagnostics, getHoverInformation, getVariablesJSONSchema, getOperationASTFacts, } from 'graphql-language-service';
import { defaultSchemaLoader } from './schemaLoader';
const schemaCache = new Map();
export class LanguageService {
    constructor({ parser, schemas, parseOptions, exteralFragmentDefinitions, customValidationRules, }) {
        this._parser = parse;
        this._schemas = [];
        this._schemaCache = schemaCache;
        this._schemaLoader = defaultSchemaLoader;
        this._parseOptions = undefined;
        this._customValidationRules = undefined;
        this._exteralFragmentDefinitionNodes = null;
        this._exteralFragmentDefinitionsString = null;
        this.getCompletion = (uri, documentText, position) => {
            const schema = this.getSchemaForFile(uri);
            if (!documentText || documentText.length < 1 || !schema?.schema) {
                return [];
            }
            return getAutocompleteSuggestions(schema.schema, documentText, position, undefined, this.getExternalFragmentDefinitions());
        };
        this.getDiagnostics = (uri, documentText, customRules) => {
            const schema = this.getSchemaForFile(uri);
            if (!documentText || documentText.length < 1 || !schema?.schema) {
                return [];
            }
            return getDiagnostics(documentText, schema.schema, customRules ?? this._customValidationRules, false, this.getExternalFragmentDefinitions());
        };
        this.getHover = (uri, documentText, position, options) => {
            const schema = this.getSchemaForFile(uri);
            if (schema && documentText?.length > 3) {
                return getHoverInformation(schema.schema, documentText, position, undefined, {
                    useMarkdown: true,
                    ...options,
                });
            }
        };
        this.getVariablesJSONSchema = (uri, documentText, options) => {
            const schema = this.getSchemaForFile(uri);
            if (schema && documentText.length > 3) {
                try {
                    const documentAST = this.parse(documentText);
                    const operationFacts = getOperationASTFacts(documentAST, schema.schema);
                    if (operationFacts && operationFacts.variableToType) {
                        return getVariablesJSONSchema(operationFacts.variableToType, options);
                    }
                }
                catch (err) { }
            }
            return null;
        };
        this._schemaLoader = defaultSchemaLoader;
        if (schemas) {
            this._schemas = schemas;
            this._cacheSchemas();
        }
        if (parser) {
            this._parser = parser;
        }
        if (parseOptions) {
            this._parseOptions = parseOptions;
        }
        if (customValidationRules) {
            this._customValidationRules = customValidationRules;
        }
        if (exteralFragmentDefinitions) {
            if (Array.isArray(exteralFragmentDefinitions)) {
                this._exteralFragmentDefinitionNodes = exteralFragmentDefinitions;
            }
            else {
                this._exteralFragmentDefinitionsString = exteralFragmentDefinitions;
            }
        }
    }
    _cacheSchemas() {
        this._schemas.forEach(schema => this._cacheSchema(schema));
    }
    _cacheSchema(schemaConfig) {
        const schema = this._schemaLoader(schemaConfig, this.parse.bind(this));
        return this._schemaCache.set(schemaConfig.uri, {
            ...schemaConfig,
            schema,
        });
    }
    getSchemaForFile(uri) {
        if (!this._schemas || !this._schemas.length) {
            return;
        }
        if (this._schemas.length === 1) {
            return this._schemaCache.get(this._schemas[0].uri);
        }
        else {
            const schema = this._schemas.find(schemaConfig => {
                if (!schemaConfig.fileMatch) {
                    return false;
                }
                return schemaConfig.fileMatch.some(glob => {
                    const isMatch = picomatch(glob);
                    return isMatch(uri);
                });
            });
            if (schema) {
                const cacheEntry = this._schemaCache.get(schema.uri);
                if (cacheEntry) {
                    return cacheEntry;
                }
                const cache = this._cacheSchema(schema);
                return cache.get(schema.uri);
            }
        }
    }
    getExternalFragmentDefinitions() {
        if (!this._exteralFragmentDefinitionNodes &&
            this._exteralFragmentDefinitionsString) {
            const definitionNodes = [];
            try {
                visit(this._parser(this._exteralFragmentDefinitionsString), {
                    FragmentDefinition(node) {
                        definitionNodes.push(node);
                    },
                });
            }
            catch (err) {
                throw Error(`Failed parsing exteralFragmentDefinitions string:\n${this._exteralFragmentDefinitionsString}`);
            }
            this._exteralFragmentDefinitionNodes = definitionNodes;
        }
        return this._exteralFragmentDefinitionNodes;
    }
    async updateSchemas(schemas) {
        this._schemas = schemas;
        this._cacheSchemas();
    }
    updateSchema(schema) {
        const schemaIndex = this._schemas.findIndex(c => c.uri === schema.uri);
        if (schemaIndex < 0) {
            console.warn('updateSchema could not find a schema in your config by that URI', schema.uri);
            return;
        }
        this._schemas[schemaIndex] = schema;
        this._cacheSchema(schema);
    }
    addSchema(schema) {
        this._schemas.push(schema);
        this._cacheSchema(schema);
    }
    parse(text, options) {
        return this._parser(text, options || this._parseOptions);
    }
}
//# sourceMappingURL=LanguageService.js.map